@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2010 Carlos Garcia Campos  <carlosgc@gnome.org>
+ * Copyright (C) 2025 Nelson Benítez León <nbenitezl@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +37,8 @@ typedef struct
     GtkWidget *fg_color_button;
     GtkWidget *bg_color_button;
     GtkWidget *copy_button;
+    GtkWidget *transparent_button;
+    GtkWidget *opacity_range;
 
     PopplerPage *page;
     cairo_surface_t *surface;
@@ -81,8 +84,9 @@ static void pgd_selections_clear_selections(PgdSelectionsDemo *demo)
 
 static void pgd_selections_free(PgdSelectionsDemo *demo)
 {
-    if (!demo)
+    if (!demo) {
         return;
+    }
 
     if (demo->selections_idle > 0) {
         g_source_remove(demo->selections_idle);
@@ -113,8 +117,9 @@ static void pgd_selections_update_selection_region(PgdSelectionsDemo *demo)
 {
     PopplerRectangle area = { 0, 0, 0, 0 };
 
-    if (demo->selection_region)
+    if (demo->selection_region) {
         cairo_region_destroy(demo->selection_region);
+    }
 
     poppler_page_get_size(demo->page, &area.x2, &area.y2);
     demo->selection_region = poppler_page_get_selected_region(demo->page, 1.0, POPPLER_SELECTION_GLYPH, &area);
@@ -124,11 +129,13 @@ static void pgd_selections_update_selected_text(PgdSelectionsDemo *demo)
 {
     gchar *text;
 
-    if (demo->selected_region)
+    if (demo->selected_region) {
         cairo_region_destroy(demo->selected_region);
+    }
     demo->selected_region = poppler_page_get_selected_region(demo->page, 1.0, demo->style, &demo->doc_area);
-    if (demo->selected_text)
+    if (demo->selected_text) {
         g_free(demo->selected_text);
+    }
     demo->selected_text = NULL;
 
     text = poppler_page_get_selected_text(demo->page, demo->style, &demo->doc_area);
@@ -147,8 +154,9 @@ static void pgd_selections_update_cursor(PgdSelectionsDemo *demo, GdkCursorType
     GdkWindow *window = gtk_widget_get_window(demo->darea);
     GdkCursor *cursor = NULL;
 
-    if (cursor_type == demo->cursor)
+    if (cursor_type == demo->cursor) {
         return;
+    }
 
     if (cursor_type != GDK_LAST_CURSOR) {
         cursor = gdk_cursor_new_for_display(gtk_widget_get_display(demo->darea), cursor_type);
@@ -158,8 +166,9 @@ static void pgd_selections_update_cursor(PgdSelectionsDemo *demo, GdkCursorType
 
     gdk_window_set_cursor(window, cursor);
     gdk_display_flush(gtk_widget_get_display(demo->darea));
-    if (cursor)
+    if (cursor) {
         g_object_unref(cursor);
+    }
 }
 
 static gboolean pgd_selections_render_selections(PgdSelectionsDemo *demo)
@@ -183,13 +192,20 @@ static gboolean pgd_selections_render_selections(PgdSelectionsDemo *demo)
     doc_area.x2 = demo->stop.x / demo->scale;
     doc_area.y2 = demo->stop.y / demo->scale;
 
-    if (demo->selection_surface)
+    if (demo->selection_surface) {
         cairo_surface_destroy(demo->selection_surface);
+    }
     demo->selection_surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, page_width, page_height);
     cr = cairo_create(demo->selection_surface);
-    if (demo->scale != 1.0)
+    if (demo->scale != 1.0) {
         cairo_scale(cr, demo->scale, demo->scale);
-    poppler_page_render_selection(demo->page, cr, &doc_area, &demo->doc_area, demo->style, &demo->glyph_color, &demo->background_color);
+    }
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(demo->transparent_button))) {
+        double opacity_val = gtk_range_get_value(GTK_RANGE(demo->opacity_range));
+        poppler_page_render_transparent_selection(demo->page, cr, &doc_area, &demo->doc_area, demo->style, &demo->background_color, opacity_val);
+    } else {
+        poppler_page_render_selection(demo->page, cr, &doc_area, &demo->doc_area, demo->style, &demo->glyph_color, &demo->background_color);
+    }
     cairo_destroy(cr);
 
     demo->doc_area = doc_area;
@@ -202,8 +218,9 @@ static gboolean pgd_selections_render_selections(PgdSelectionsDemo *demo)
 
 static gboolean pgd_selections_drawing_area_draw(GtkWidget *area, cairo_t *cr, PgdSelectionsDemo *demo)
 {
-    if (!demo->surface)
+    if (!demo->surface) {
         return FALSE;
+    }
 
     cairo_save(cr);
     cairo_set_source_surface(cr, demo->surface, 0, 0);
@@ -220,11 +237,13 @@ static gboolean pgd_selections_drawing_area_draw(GtkWidget *area, cairo_t *cr, P
 
 static gboolean pgd_selections_drawing_area_button_press(GtkWidget *area, GdkEventButton *event, PgdSelectionsDemo *demo)
 {
-    if (!demo->page)
+    if (!demo->page) {
         return FALSE;
+    }
 
-    if (event->button != 1)
+    if (event->button != 1) {
         return FALSE;
+    }
 
     demo->start.x = event->x;
     demo->start.y = event->y;
@@ -248,8 +267,9 @@ static gboolean pgd_selections_drawing_area_button_press(GtkWidget *area, GdkEve
 
 static gboolean pgd_selections_drawing_area_motion_notify(GtkWidget *area, GdkEventMotion *event, PgdSelectionsDemo *demo)
 {
-    if (!demo->page)
+    if (!demo->page) {
         return FALSE;
+    }
 
     if (demo->start.x != -1) {
         demo->stop.x = event->x;
@@ -269,14 +289,17 @@ static gboolean pgd_selections_drawing_area_motion_notify(GtkWidget *area, GdkEv
 
 static gboolean pgd_selections_drawing_area_button_release(GtkWidget *area, GdkEventButton *event, PgdSelectionsDemo *demo)
 {
-    if (!demo->page)
+    if (!demo->page) {
         return FALSE;
+    }
 
-    if (event->button != 1)
+    if (event->button != 1) {
         return FALSE;
+    }
 
-    if (demo->start.x != -1)
+    if (demo->start.x != -1) {
         pgd_selections_update_selected_text(demo);
+    }
 
     demo->start.x = -1;
 
@@ -306,8 +329,9 @@ static gboolean pgd_selections_drawing_area_query_tooltip(GtkWidget *area, gint
 {
     gboolean over_selection;
 
-    if (!demo->selected_text)
+    if (!demo->selected_text) {
         return FALSE;
+    }
 
     over_selection = cairo_region_contains_point(demo->selected_region, x / demo->scale, y / demo->scale);
 
@@ -334,18 +358,21 @@ static void pgd_selections_render(GtkButton *button, PgdSelectionsDemo *demo)
     gdouble page_width, page_height;
     cairo_t *cr;
 
-    if (!demo->page)
+    if (!demo->page) {
         demo->page = poppler_document_get_page(demo->doc, demo->page_index);
+    }
 
-    if (!demo->page)
+    if (!demo->page) {
         return;
+    }
 
     pgd_selections_clear_selections(demo);
     pgd_selections_update_selection_region(demo);
     gtk_widget_set_sensitive(demo->copy_button, FALSE);
 
-    if (demo->surface)
+    if (demo->surface) {
         cairo_surface_destroy(demo->surface);
+    }
     demo->surface = NULL;
 
     poppler_page_get_size(demo->page, &page_width, &page_height);
@@ -357,8 +384,9 @@ static void pgd_selections_render(GtkButton *button, PgdSelectionsDemo *demo)
 
     cairo_save(cr);
 
-    if (demo->scale != 1.0)
+    if (demo->scale != 1.0) {
         cairo_scale(cr, demo->scale, demo->scale);
+    }
 
     poppler_page_render(demo->page, cr);
     cairo_restore(cr);
@@ -382,8 +410,9 @@ static void pgd_selections_copy(GtkButton *button, PgdSelectionsDemo *demo)
 static void pgd_selections_page_selector_value_changed(GtkSpinButton *spinbutton, PgdSelectionsDemo *demo)
 {
     demo->page_index = (gint)gtk_spin_button_get_value(spinbutton) - 1;
-    if (demo->page)
+    if (demo->page) {
         g_object_unref(demo->page);
+    }
     demo->page = NULL;
 }
 
@@ -522,6 +551,18 @@ GtkWidget *pgd_selections_properties_selector_create(PgdSelectionsDemo *demo)
     gtk_box_pack_start(GTK_BOX(hbox), color_hbox, FALSE, TRUE, 0);
     gtk_widget_show(color_hbox);
 
+    demo->transparent_button = gtk_check_button_new_with_label("Transparent");
+    gtk_box_pack_start(GTK_BOX(hbox), demo->transparent_button, TRUE, TRUE, 0);
+    gtk_widget_show(demo->transparent_button);
+
+    demo->opacity_range = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0., 1., 0.1);
+    gtk_box_pack_start(GTK_BOX(hbox), demo->opacity_range, TRUE, TRUE, 0);
+    gtk_widget_show(demo->opacity_range);
+    g_object_bind_property(demo->transparent_button, "active", demo->opacity_range, "sensitive", G_BINDING_SYNC_CREATE);
+
+    gtk_box_pack_start(GTK_BOX(hbox), color_hbox, FALSE, TRUE, 0);
+    gtk_widget_show(color_hbox);
+
     demo->copy_button = gtk_button_new_with_label("Copy");
     g_signal_connect(G_OBJECT(demo->copy_button), "clicked", G_CALLBACK(pgd_selections_copy), (gpointer)demo);
     gtk_box_pack_end(GTK_BOX(hbox), demo->copy_button, FALSE, TRUE, 0);
@@ -564,11 +605,7 @@ GtkWidget *pgd_selections_create_widget(PopplerDocument *document)
     g_signal_connect(demo->darea, "query_tooltip", G_CALLBACK(pgd_selections_drawing_area_query_tooltip), (gpointer)demo);
     demo->swindow = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(demo->swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#if GTK_CHECK_VERSION(3, 7, 8)
     gtk_container_add(GTK_CONTAINER(demo->swindow), demo->darea);
-#else
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(demo->swindow), demo->darea);
-#endif
     gtk_widget_show(demo->darea);
 
     gtk_box_pack_start(GTK_BOX(vbox), demo->swindow, TRUE, TRUE, 0);
