@@ -19,6 +19,7 @@
 // Copyright (C) 2013, 2017, 2019, 2022 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018, 2019 Adam Reichold <adam.reichold@t-online.de>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -87,7 +88,7 @@ void Array::remove(int i)
 Object Array::get(int i, int recursion) const
 {
     if (i < 0 || std::size_t(i) >= elems.size()) {
-        return Object(objNull);
+        return Object::null();
     }
     return elems[i].fetch(xref, recursion);
 }
@@ -96,7 +97,7 @@ Object Array::get(int i, Ref *returnRef, int recursion) const
 {
     if (i < 0 || std::size_t(i) >= elems.size()) {
         *returnRef = Ref::INVALID();
-        return Object(objNull);
+        return Object::null();
     }
     if (elems[i].getType() == objRef) {
         *returnRef = elems[i].getRef();
@@ -109,7 +110,7 @@ Object Array::get(int i, Ref *returnRef, int recursion) const
 const Object &Array::getNF(int i) const
 {
     if (i < 0 || std::size_t(i) >= elems.size()) {
-        static Object nullObj(objNull);
+        static Object nullObj = Object::null();
         return nullObj;
     }
     return elems[i];
