@@ -17,6 +17,7 @@
 // Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2020 Jakub Alba <jakubalba@gmail.com>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -49,7 +50,7 @@ Object Object::copy() const
     switch (type) {
     case objString:
     case objHexString:
-        obj.string = new GooString(string);
+        obj.string = string->copy().release();
         break;
     case objName:
     case objCmd:
@@ -81,7 +82,7 @@ Object Object::deepCopy() const
     switch (type) {
     case objString:
     case objHexString:
-        obj.string = new GooString(string);
+        obj.string = string->copy().release();
         break;
     case objName:
     case objCmd:
@@ -150,8 +151,6 @@ const char *Object::getTypeName() const
 
 void Object::print(FILE *f) const
 {
-    int i;
-
     switch (type) {
     case objBool:
         fprintf(f, "%s", booln ? "true" : "false");
@@ -164,12 +163,12 @@ void Object::print(FILE *f) const
         break;
     case objString:
         fprintf(f, "(");
-        fwrite(string->c_str(), 1, string->getLength(), f);
+        fwrite(string->c_str(), 1, string->size(), f);
         fprintf(f, ")");
         break;
     case objHexString:
         fprintf(f, "<");
-        for (i = 0; i < string->getLength(); i++) {
+        for (size_t i = 0; i < string->size(); i++) {
             fprintf(f, "%02x", string->getChar(i) & 0xff);
         }
         fprintf(f, ">");
@@ -182,7 +181,7 @@ void Object::print(FILE *f) const
         break;
     case objArray:
         fprintf(f, "[");
-        for (i = 0; i < arrayGetLength(); ++i) {
+        for (int i = 0; i < arrayGetLength(); ++i) {
             if (i > 0) {
                 fprintf(f, " ");
             }
@@ -193,7 +192,7 @@ void Object::print(FILE *f) const
         break;
     case objDict:
         fprintf(f, "<<");
-        for (i = 0; i < dictGetLength(); ++i) {
+        for (int i = 0; i < dictGetLength(); ++i) {
             fprintf(f, " /%s ", dictGetKey(i));
             const Object &obj = dictGetValNF(i);
             obj.print(f);
