@@ -19,6 +19,7 @@
  * Copyright (C) 2023-2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  * Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
  * Copyright (C) 2024 Stefan Brüns <stefan.bruens@rwth-aachen.de>
+ * Copyright (C) 2025 Blair Bonnett <blair.bonnett@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +37,7 @@
  */
 
 #include "poppler-form.h"
+#include "poppler-converter.h"
 
 #include <config.h>
 
@@ -53,7 +55,7 @@
 #    include <NSSCryptoSignBackend.h>
 #endif
 #ifdef ENABLE_GPGME
-#    include <GPGMECryptoSignBackend.h>
+#    include <GPGMECryptoSignBackendConfiguration.h>
 #endif
 
 #include "poppler-page-private.h"
@@ -271,7 +273,7 @@ std::unique_ptr<Link> FormField::additionalAction(AdditionalActionType type) con
 
 std::unique_ptr<Link> FormField::additionalAction(Annotation::AdditionalActionType type) const
 {
-    ::AnnotWidget *w = m_formData->fm->getWidgetAnnotation();
+    ::AnnotWidget *w = m_formData->fm->getWidgetAnnotation().get();
     if (!w) {
         return {};
     }
@@ -352,7 +354,7 @@ void FormFieldButton::setIcon(const FormFieldIcon &icon)
 
     FormWidgetButton *fwb = static_cast<FormWidgetButton *>(m_formData->fm);
     if (fwb->getButtonType() == formButtonPush) {
-        ::AnnotWidget *w = m_formData->fm->getWidgetAnnotation();
+        std::shared_ptr<::AnnotWidget> w = m_formData->fm->getWidgetAnnotation();
         FormFieldIconData *data = FormFieldIconData::getData(icon);
         if (data->icon != nullptr) {
             w->setNewAppearance(data->icon->lookup("AP"));
@@ -816,7 +818,7 @@ bool CertificateInfo::checkPassword(const QString &password) const
     unsigned char buffer[5];
     memcpy(buffer, "test", 5);
     sigHandler->addData(buffer, 5);
-    std::variant<std::vector<unsigned char>, CryptoSign::SigningError> tmpSignature = sigHandler->signDetached(password.toStdString());
+    std::variant<std::vector<unsigned char>, CryptoSign::SigningErrorMessage> tmpSignature = sigHandler->signDetached(password.toStdString());
     return std::holds_alternative<std::vector<unsigned char>>(tmpSignature);
 #else
     return false;
@@ -1043,33 +1045,33 @@ static CertificateInfoPrivate *createCertificateInfoPrivate(const X509Certificat
         certPriv->certificateType = fromPopplerCore(ci->getCertificateType());
 
         const GooString &certSerial = ci->getSerialNumber();
-        certPriv->serial_number = QByteArray(certSerial.c_str(), certSerial.getLength());
+        certPriv->serial_number = QByteArray(certSerial.c_str(), certSerial.size());
 
         const X509CertificateInfo::EntityInfo &issuerInfo = ci->getIssuerInfo();
-        certPriv->issuer_info.common_name = issuerInfo.commonName.c_str();
-        certPriv->issuer_info.distinguished_name = issuerInfo.distinguishedName.c_str();
-        certPriv->issuer_info.email_address = issuerInfo.email.c_str();
-        certPriv->issuer_info.org_name = issuerInfo.organization.c_str();
+        certPriv->issuer_info.common_name = QString::fromStdString(issuerInfo.commonName);
+        certPriv->issuer_info.distinguished_name = QString::fromStdString(issuerInfo.distinguishedName);
+        certPriv->issuer_info.email_address = QString::fromStdString(issuerInfo.email);
+        certPriv->issuer_info.org_name = QString::fromStdString(issuerInfo.organization);
 
         const X509CertificateInfo::EntityInfo &subjectInfo = ci->getSubjectInfo();
-        certPriv->subject_info.common_name = subjectInfo.commonName.c_str();
-        certPriv->subject_info.distinguished_name = subjectInfo.distinguishedName.c_str();
-        certPriv->subject_info.email_address = subjectInfo.email.c_str();
-        certPriv->subject_info.org_name = subjectInfo.organization.c_str();
+        certPriv->subject_info.common_name = QString::fromStdString(subjectInfo.commonName);
+        certPriv->subject_info.distinguished_name = QString::fromStdString(subjectInfo.distinguishedName);
+        certPriv->subject_info.email_address = QString::fromStdString(subjectInfo.email);
+        certPriv->subject_info.org_name = QString::fromStdString(subjectInfo.organization);
 
-        certPriv->nick_name = ci->getNickName().c_str();
+        certPriv->nick_name = QString::fromStdString(ci->getNickName().toStr());
 
         X509CertificateInfo::Validity certValidity = ci->getValidity();
         certPriv->validity_start = QDateTime::fromSecsSinceEpoch(certValidity.notBefore, QTimeZone::utc());
         certPriv->validity_end = QDateTime::fromSecsSinceEpoch(certValidity.notAfter, QTimeZone::utc());
 
         const X509CertificateInfo::PublicKeyInfo &pkInfo = ci->getPublicKeyInfo();
-        certPriv->public_key = QByteArray(pkInfo.publicKey.c_str(), pkInfo.publicKey.getLength());
+        certPriv->public_key = QByteArray(pkInfo.publicKey.c_str(), pkInfo.publicKey.size());
         certPriv->public_key_type = static_cast<int>(pkInfo.publicKeyType);
         certPriv->public_key_strength = pkInfo.publicKeyStrength;
 
         const GooString &certDer = ci->getCertificateDER();
-        certPriv->certificate_der = QByteArray(certDer.c_str(), certDer.getLength());
+        certPriv->certificate_der = QByteArray(certDer.c_str(), certDer.size());
 
         certPriv->is_null = false;
         certPriv->is_qualified = ci->isQualified();
@@ -1144,9 +1146,10 @@ static SignatureValidationInfo fromInternal(SignatureInfo *si, FormWidgetSignatu
             priv->range_bounds.append(bound);
         }
     }
-    const std::optional<GooString> checkedSignature = fws->getCheckedSignature(&priv->docLength);
+    std::optional<std::vector<unsigned char>> checkedSignature;
+    std::tie(checkedSignature, priv->docLength) = fws->getCheckedSignature();
     if (priv->range_bounds.size() == 4 && checkedSignature) {
-        priv->signature = QByteArray::fromHex(checkedSignature->c_str());
+        priv->signature = QByteArray::fromRawData(reinterpret_cast<const char *>(checkedSignature->data()), checkedSignature->size());
     }
 
     return SignatureValidationInfo(priv);
@@ -1181,7 +1184,7 @@ std::pair<SignatureValidationInfo, std::shared_ptr<Poppler::AsyncObject>> FormFi
                                                                     l.get(),
                                                                     [innerObj = std::weak_ptr<AsyncObject>(l)]() {
                                                                         if (auto innerLocked = innerObj.lock()) {
-                                                                            emit innerLocked->done();
+                                                                            Q_EMIT innerLocked->done();
                                                                         }
                                                                     },
                                                                     Qt::QueuedConnection);
@@ -1203,8 +1206,7 @@ FormFieldSignature::SigningResult FormFieldSignature::sign(const QString &output
         return FieldAlreadySigned;
     }
 
-    Goffset file_size = 0;
-    const std::optional<GooString> sig = fws->getCheckedSignature(&file_size);
+    const auto [sig, file_size] = fws->getCheckedSignature();
     if (sig) {
         // the above unsigned_signature_field check
         // should already catch this, but double check
@@ -1220,7 +1222,8 @@ FormFieldSignature::SigningResult FormFieldSignature::sign(const QString &output
     const auto failure = fws->signDocumentWithAppearance(outputFileName.toStdString(), data.certNickname().toStdString(), data.password().toStdString(), reason.get(), location.get(), ownerPwd, userPwd, *gSignatureText, *gSignatureLeftText,
                                                          data.fontSize(), data.leftFontSize(), convertQColor(data.fontColor()), data.borderWidth(), convertQColor(data.borderColor()), convertQColor(data.backgroundColor()));
     if (failure) {
-        switch (failure.value()) {
+        m_formData->lastSigningErrorDetails = fromPopplerCore(failure.value().message);
+        switch (failure.value().type) {
         case CryptoSign::SigningError::GenericError:
             return GenericSigningError;
         case CryptoSign::SigningError::InternalError:
@@ -1234,10 +1237,17 @@ FormFieldSignature::SigningResult FormFieldSignature::sign(const QString &output
         case CryptoSign::SigningError::BadPassphrase:
             return BadPassphrase;
         }
+        return GenericSigningError;
     }
+    m_formData->lastSigningErrorDetails = {};
     return SigningSuccess;
 }
 
+ErrorString FormFieldSignature::lastSigningErrorDetails() const
+{
+    return m_formData->lastSigningErrorDetails;
+}
+
 bool hasNSSSupport()
 {
 #ifdef ENABLE_NSS3
