@@ -1,4 +1,4 @@
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
 
 #include <poppler-qt6.h>
 
@@ -6,8 +6,8 @@ class TestPageMode : public QObject
 {
     Q_OBJECT
 public:
-    TestPageMode(QObject *parent = nullptr) : QObject(parent) { }
-private slots:
+    explicit TestPageMode(QObject *parent = nullptr) : QObject(parent) { }
+private Q_SLOTS:
     void checkNone();
     void checkFullScreen();
     void checkAttachments();
@@ -17,7 +17,7 @@ private slots:
 
 void TestPageMode::checkNone()
 {
-    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/UseNone.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/UseNone.pdf"));
     QVERIFY(doc);
 
     QCOMPARE(doc->pageMode(), Poppler::Document::UseNone);
@@ -25,7 +25,7 @@ void TestPageMode::checkNone()
 
 void TestPageMode::checkFullScreen()
 {
-    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/FullScreen.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/FullScreen.pdf"));
     QVERIFY(doc);
 
     QCOMPARE(doc->pageMode(), Poppler::Document::FullScreen);
@@ -33,7 +33,7 @@ void TestPageMode::checkFullScreen()
 
 void TestPageMode::checkAttachments()
 {
-    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/UseAttachments.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/UseAttachments.pdf"));
     QVERIFY(doc);
 
     QCOMPARE(doc->pageMode(), Poppler::Document::UseAttach);
@@ -41,7 +41,7 @@ void TestPageMode::checkAttachments()
 
 void TestPageMode::checkThumbs()
 {
-    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/UseThumbs.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/UseThumbs.pdf"));
     QVERIFY(doc);
 
     QCOMPARE(doc->pageMode(), Poppler::Document::UseThumbs);
@@ -49,7 +49,7 @@ void TestPageMode::checkThumbs()
 
 void TestPageMode::checkOC()
 {
-    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/UseOC.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/UseOC.pdf"));
     QVERIFY(doc);
 
     QCOMPARE(doc->pageMode(), Poppler::Document::UseOC);
