@@ -11,7 +11,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2006, 2009, 2010, 2012, 2015, 2018, 2019, 2021, 2022 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2009, 2010, 2012, 2015, 2018, 2019, 2021, 2022, 2024 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2007 Ilmari Heikkinen <ilmari.heikkinen@gmail.com>
 // Copyright (C) 2009 Shen Liang <shenzhuxi@gmail.com>
 // Copyright (C) 2009 Stefan Thomas <thomas@eload24.com>
@@ -26,6 +26,7 @@
 // Copyright (C) 2018 Martin Packman <gzlist@googlemail.com>
 // Copyright (C) 2019 Christian Persch <chpe@src.gnome.org>
 // Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -43,6 +44,7 @@
 #include "SplashErrorCodes.h"
 #include "SplashBitmap.h"
 #include "poppler/Error.h"
+#include "poppler/GfxState.h"
 #include "goo/JpegWriter.h"
 #include "goo/PNGWriter.h"
 #include "goo/TiffWriter.h"
@@ -52,12 +54,13 @@
 // SplashBitmap
 //------------------------------------------------------------------------
 
-SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPadA, SplashColorMode modeA, bool alphaA, bool topDown, const std::vector<GfxSeparationColorSpace *> *separationListA)
+SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPadA, SplashColorMode modeA, bool alphaA, bool topDown, const std::vector<std::unique_ptr<GfxSeparationColorSpace>> *separationListA)
 {
     width = widthA;
     height = heightA;
     mode = modeA;
     rowPad = rowPadA;
+    rowSize = -1; // some compilers can't see it will be initialized
     switch (mode) {
     case splashModeMono1:
         if (width > 0) {
@@ -114,17 +117,17 @@ SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPadA, SplashColorMode
             rowSize = -rowSize;
         }
         if (alphaA) {
-            alpha = (unsigned char *)gmallocn(width, height);
+            alpha = (unsigned char *)gmallocn_checkoverflow(width, height);
         } else {
             alpha = nullptr;
         }
     } else {
         alpha = nullptr;
     }
-    separationList = new std::vector<GfxSeparationColorSpace *>();
+    separationList = new std::vector<std::unique_ptr<GfxSeparationColorSpace>>();
     if (separationListA != nullptr) {
-        for (const GfxSeparationColorSpace *separation : *separationListA) {
-            separationList->push_back((GfxSeparationColorSpace *)separation->copy());
+        for (const std::unique_ptr<GfxSeparationColorSpace> &separation : *separationListA) {
+            separationList->push_back(separation->copyAsOwnType());
         }
     }
 }
@@ -159,9 +162,6 @@ SplashBitmap::~SplashBitmap()
         }
     }
     gfree(alpha);
-    for (auto entry : *separationList) {
-        delete entry;
-    }
     delete separationList;
 }
 
@@ -446,13 +446,13 @@ void SplashBitmap::getRGBLine(int yl, SplashColorPtr line)
         m = byteToDbl(col[1]);
         y = byteToDbl(col[2]);
         k = byteToDbl(col[3]);
-        if (separationList->size() > 0) {
+        if (!separationList->empty()) {
             for (std::size_t i = 0; i < separationList->size(); i++) {
                 if (col[i + 4] > 0) {
                     GfxCMYK cmyk;
                     GfxColor input;
                     input.c[0] = byteToCol(col[i + 4]);
-                    GfxSeparationColorSpace *sepCS = (GfxSeparationColorSpace *)((*separationList)[i]);
+                    const std::unique_ptr<GfxSeparationColorSpace> &sepCS = (*separationList)[i];
                     sepCS->getCMYK(&input, &cmyk);
                     col[0] = colToByte(cmyk.c);
                     col[1] = colToByte(cmyk.m);
@@ -499,13 +499,13 @@ void SplashBitmap::getXBGRLine(int yl, SplashColorPtr line, ConversionMode conve
         m = byteToDbl(col[1]);
         y = byteToDbl(col[2]);
         k = byteToDbl(col[3]);
-        if (separationList->size() > 0) {
+        if (!separationList->empty()) {
             for (std::size_t i = 0; i < separationList->size(); i++) {
                 if (col[i + 4] > 0) {
                     GfxCMYK cmyk;
                     GfxColor input;
                     input.c[0] = byteToCol(col[i + 4]);
-                    GfxSeparationColorSpace *sepCS = (GfxSeparationColorSpace *)((*separationList)[i]);
+                    const std::unique_ptr<GfxSeparationColorSpace> &sepCS = (*separationList)[i];
                     sepCS->getCMYK(&input, &cmyk);
                     col[0] = colToByte(cmyk.c);
                     col[1] = colToByte(cmyk.m);
@@ -617,7 +617,7 @@ void SplashBitmap::getCMYKLine(int yl, SplashColorPtr line)
 
     for (int x = 0; x < width; x++) {
         getPixel(x, yl, col);
-        if (separationList->size() > 0) {
+        if (!separationList->empty()) {
             double c, m, y, k;
             c = byteToDbl(col[0]);
             m = byteToDbl(col[1]);
@@ -628,7 +628,7 @@ void SplashBitmap::getCMYKLine(int yl, SplashColorPtr line)
                     GfxCMYK cmyk;
                     GfxColor input;
                     input.c[0] = byteToCol(col[i + 4]);
-                    GfxSeparationColorSpace *sepCS = (GfxSeparationColorSpace *)((*separationList)[i]);
+                    const std::unique_ptr<GfxSeparationColorSpace> &sepCS = (*separationList)[i];
                     sepCS->getCMYK(&input, &cmyk);
                     col[0] = colToByte(cmyk.c);
                     col[1] = colToByte(cmyk.m);
